/*
 * IRC - Internet Relay Chat, include/runmalloc.h
 * (C) Copyright 1996 - 1997, Carlo Wood (carlo@runaway.xs4all.nl)
 *
 * See file doc/authors for names of the additional programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef RUNMALLOC_H
#define RUNMALLOC_H

#ifdef DEBUGMALLOC

#if defined(MEMMAGICNUMS) && !defined(MEMSIZESTATS)
#define MEMSIZESTATS
#endif
#ifndef MEMLEAKSTATS
#undef MEMTIMESTATS
#endif

/*=============================================================================
 * Proto types
 */

#ifdef MEMLEAKSTATS
extern void *RunMalloc_memleak(size_t size, int line, const char *filename);
extern void *RunCalloc_memleak(size_t nmemb, size_t size,
    int line, const char *filename);
extern void *RunRealloc_memleak(void *ptr, size_t size,
    int line, const char *filename);
struct Client;
extern void report_memleak_stats(struct Client *sptr, int parc, char *parv[]);
#define RunMalloc(x) RunMalloc_memleak(x, __LINE__, __FILE__)
#define RunCalloc(x,y) RunCalloc_memleak(x,y, __LINE__, __FILE__)
#define RunRealloc(x,y) RunRealloc_memleak(x,y, __LINE__, __FILE__)
#else
extern void *RunMalloc(size_t size);
extern void *RunCalloc(size_t nmemb, size_t size);
extern void *RunRealloc(void *ptr, size_t size);
#endif
extern int RunFree_test(void *ptr);
extern void RunFree(void *ptr);
#ifdef MEMSIZESTATS
extern unsigned int get_alloc_cnt(void);
extern size_t get_mem_size(void);
#endif

#else /* !DEBUGMALLOC */

#include <stdlib.h>

#undef MEMSIZESTATS
#undef MEMMAGICNUMS
#undef MEMLEAKSTATS
#undef MEMTIMESTATS

#define Debug_malloc(x)
#define RunMalloc(x) malloc(x)
#define RunCalloc(x,y) calloc(x,y)
#define RunRealloc(x,y) realloc(x,y)
#define RunFree(x) free(x)

#endif /* DEBUGMALLOC */

#endif /* RUNMALLOC_H */
