/*
 * IRC - Internet Relay Chat, include/s_bsd.h
 *
 * See file doc/authors for names of the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef S_BSD_H
#define S_BSD_H

#include <netdb.h>
#include "s_conf.h"

/*=============================================================================
 * Macro's
 */

#define FLAGS_PINGSENT	 0x0001	/* Unreplied ping sent */
#define FLAGS_DEADSOCKET 0x0002	/* Local socket is dead--Exiting soon */
#define FLAGS_KILLED	 0x0004	/* Prevents "QUIT" from being sent for this */
#define FLAGS_OPER	 0x0008	/* Operator */
#define FLAGS_LOCOP	 0x0010	/* Local operator -- SRB */
#define FLAGS_INVISIBLE	 0x0020	/* makes user invisible */
#define FLAGS_WALLOP	 0x0040	/* send wallops to them */
#define FLAGS_SERVNOTICE 0x0080	/* server notices such as kill */
#define FLAGS_BLOCKED	 0x0100	/* socket is in a blocked condition */
#define FLAGS_UNIX	 0x0200	/* socket is in the unix domain, not inet */
#define FLAGS_CLOSING	 0x0400	/* set when closing to suppress errors */
#define FLAGS_LISTEN	 0x0800	/* used to mark clients which we listen() on */
#define FLAGS_CHKACCESS	 0x1000	/* ok to check clients access if set */
#define FLAGS_DOINGDNS	 0x2000	/* client is waiting for a DNS response */
#define FLAGS_AUTH	 0x4000	/* client is waiting on rfc931 response */
#define FLAGS_WRAUTH	 0x8000	/* set if we havent writen to ident server */
#define FLAGS_LOCAL	0x00010000	/* set for local clients */
#define FLAGS_GOTID	0x00020000	/* successful ident lookup achieved */
#define FLAGS_DOID	0x00040000	/* I-lines say must use ident return */
#define FLAGS_NONL	0x00080000	/* No \n in buffer */
#define FLAGS_TS8      	0x00100000	/* Why do you want to know? */
#define FLAGS_PING	0x00200000	/* Socket needs to send udp pings */
#define FLAGS_ASKEDPING	0x00400000	/* Client asked for udp ping */
#define FLAGS_MAP	0x00800000	/* Show server on the map */
#define FLAGS_JUNCTION	0x01000000	/* Junction causing the net.burst */
#define FLAGS_DEAF	0x02000000	/* Makes user deaf */
#define FLAGS_CHSERV	0x04000000	/* Disallow KICK or MODE -o on the user;
					   don't display channels in /whois */
#define FLAGS_BURST	0x08000000	/* Server is receiving a net.burst */
#define FLAGS_BURST_ACK	0x10000000	/* Server is waiting for eob ack */
#define FLAGS_DEBUG	0x20000000	/* send global debug/anti-hack info */
#define FLAGS_IPCHECK	0x40000000	/* Added or updated IPregistry data */

/*                         1234567812345678 */
#define FLAGS_NOPRIVATES 0x0000000100000000	/* RaptorNet's flag to disable incoming privates*/
#define FLAGS_NOCTCPDCC  0x0000000200000000	/* RaptorNet's flag to disable DCC and CTCP */
#define FLAGS_REGGED     0x0000000800000000	/* Registered nick */
#define FLAGS_HIDEHOST   0x0000001000000000	/* Flag to enable host hiding */
#define FLAGS_GODMODE    0x0000002000000000	/* AfterNET's "God" Mode for IRCops */
#define FLAGS_BOT        0x0000004000000000	/* Bot flag to indicate BitchX and eggdrop */
#define FLAGS_SBOT       0x0000008000000000	/* ServicesBot */
#define FLAGS_RCENTRAL   0x0000010000000000	/* Raptor-Central member */
#define FLAGS_ADMIN      0x0000020000000000	/* Server Admin */
#define FLAGS_COADMIN    0x0000040000000000	/* Server CoAdmin */
#define FLAGS_BACKUPOP   0x0000080000000000	/* Backup ircop */
#define FLAGS_CODER      0x0000100000000000	/* Coder-Com member */
#define FLAGS_HELPER     0x0000200000000000	/* HelpDesk member */
#define FLAGS_USERCOM    0x0000400000000000	/* User-Com member */	
#define FLAGS_WEBCOM     0x0000800000000000	/* Web-Com member */
#define FLAGS_BSERVICE   0x0001000000000000	/* BService member */
#define FLAGS_CSERVICE   0x0002000000000000	/* CService member */
#define FLAGS_ESERVICE   0x0004000000000000	/* Egg-Com member */
#define FLAGS_WHOIS      0x0008000000000000	/* wants to see /whois notice */
#define FLAGS_SOCKS      0x1000000000000000	/* check socks */
#define FLAGS_WRSOCKS    0x2000000000000000	/* write to socks */
#define FLAGS_VLINED     0x4000000000000000	/* user has a vline */


#define SEND_UMODES \
    (0x000FFFFF00000000|FLAGS_INVISIBLE|FLAGS_OPER\
    |FLAGS_WALLOP|FLAGS_DEAF|FLAGS_CHSERV|FLAGS_DEBUG)
#define ALL_UMODES (SEND_UMODES|FLAGS_SERVNOTICE|FLAGS_LOCOP)
#define FLAGS_ID (FLAGS_DOID|FLAGS_GOTID)


/*
 * flags macros.
 */
#define IsOper(x)		((x)->flags & FLAGS_OPER)
#define IsGod(x)		((x)->flags & FLAGS_GODMODE)
#define IsLocOp(x)		((x)->flags & FLAGS_LOCOP)
#define IsInvisible(x)		((x)->flags & FLAGS_INVISIBLE)
#define IsDeaf(x)		((x)->flags & FLAGS_DEAF)
#define IsChannelService(x)	((x)->flags & FLAGS_CHSERV)
#define IsNoKick(x)		((x)->flags & FLAGS_CHSERV)
#define IsAnOper(x)		((x)->flags & (FLAGS_OPER|FLAGS_LOCOP))
#define IsPrivileged(x)		(IsAnOper(x) || IsServer(x))
#define SendWallops(x)		((x)->flags & FLAGS_WALLOP)
#define SendDebug(x)            ((x)->flags & FLAGS_DEBUG)
#define SendServNotice(x)	((x)->flags & FLAGS_SERVNOTICE)
#define IsUnixSocket(x)		((x)->flags & FLAGS_UNIX)
#define IsListening(x)		((x)->flags & FLAGS_LISTEN)
#define DoAccess(x)		((x)->flags & FLAGS_CHKACCESS)
#define IsLocal(x)		((x)->flags & FLAGS_LOCAL)
#define IsDead(x)		((x)->flags & FLAGS_DEADSOCKET)
#define IsJunction(x)		((x)->flags & FLAGS_JUNCTION)
#define IsBurst(x)		((x)->flags & FLAGS_BURST)
#define IsBurstAck(x)		((x)->flags & FLAGS_BURST_ACK)
#define IsBurstOrBurstAck(x)	((x)->flags & (FLAGS_BURST|FLAGS_BURST_ACK))
#define IsIPChecked(x)		((x)->flags & FLAGS_IPCHECK)
#define AskedPing(x)		((x)->flags & FLAGS_ASKEDPING)
#define DoingDNS(x)		((x)->flags & FLAGS_DOINGDNS)
#define DoingAuth(x)		((x)->flags & FLAGS_AUTH)
#define DoingSocks(x)		((x)->flags & FLAGS_SOCKS)
#define DoingWrSocks(x)		((x)->flags & FLAGS_WRSOCKS)
#define NoNewLine(x)		((x)->flags & FLAGS_NONL)
#define DoPing(x)		((x)->flags & FLAGS_PING)
#define IsBot(x)		((x)->flags & FLAGS_BOT)
#define NoPrivates(x)		((x)->flags & FLAGS_NOPRIVATES)
#define NoCTCPDCC(x)		((x)->flags & FLAGS_NOCTCPDCC)
#define IsHideHost(x)		((x)->flags & FLAGS_HIDEHOST)
#define IsCoAdmin(x)		((x)->flags & FLAGS_COADMIN)
#define IsAdmin(x)		((x)->flags & FLAGS_ADMIN)
#define IsUserCom(x)		((x)->flags & FLAGS_USERCOM)
#define IsEService(x)		((x)->flags & FLAGS_ESERVICE)
#define IsBService(x)		((x)->flags & FLAGS_BSERVICE)
#define IsCService(x)		((x)->flags & FLAGS_CSERVICE)
#define IsSBot(x)		((x)->flags & FLAGS_SBOT)
#define IsCoder(x)		((x)->flags & FLAGS_CODER)
#define IsHelper(x)		((x)->flags & FLAGS_HELPER)
#define IsRCentral(x)		((x)->flags & FLAGS_RCENTRAL)
#define IsBackupOp(x)		((x)->flags & FLAGS_BACKUPOP)
#define IsWebCom(x)		((x)->flags & FLAGS_WEBCOM)
#define IsRegged(x)		((x)->flags & FLAGS_REGGED)
#define IsWhoisNotice(x)	((x)->flags & FLAGS_WHOIS)
#define IsVlined(x)		((x)->flags & FLAGS_VLINED)

#define SetOper(x)		((x)->flags |= FLAGS_OPER)
#define SetGod(x)		((x)->flags |= FLAGS_GODMODE)
#define SetLocOp(x)		((x)->flags |= FLAGS_LOCOP)
#define SetInvisible(x)		((x)->flags |= FLAGS_INVISIBLE)
#define SetWallops(x)		((x)->flags |= FLAGS_WALLOP)
#define SetDebug(x)             ((x)->flags |= FLAGS_DEBUG)
#define SetUnixSock(x)		((x)->flags |= FLAGS_UNIX)
#define SetDNS(x)		((x)->flags |= FLAGS_DOINGDNS)
#define SetAuth(x)		((x)->flags |= FLAGS_AUTH)
#define SetSocks(x)		((x)->flags |= FLAGS_SOCKS)
#define SetWrSocks(x)		((x)->flags |= FLAGS_WRSOCKS)
#define SetAccess(x)		((x)->flags |= FLAGS_CHKACCESS)
#define SetAskedPing(x)		((x)->flags |= FLAGS_ASKEDPING)
#define SetJunction(x)		((x)->flags |= FLAGS_JUNCTION)
#define SetBurst(x)		((x)->flags |= FLAGS_BURST)
#define SetBurstAck(x)		((x)->flags |= FLAGS_BURST_ACK)
#define SetIPChecked(x)		((x)->flags |= FLAGS_IPCHECK)
#define SetBot(x)		((x)->flags |= FLAGS_BOT)
#define SetNoPriv(x)		((x)->flags |= FLAGS_NOPRIVATES)
#define SetNoCTCP(x)		((x)->flags |= FLAGS_NOCTCPDCC)
#define SetHideHost(x)		((x)->flags |= FLAGS_HIDEHOST)
#define SetCoAdmin(x)		((x)->flags |= FLAGS_COADMIN)
#define SetAdmin(x)		((x)->flags |= FLAGS_ADMIN)
#define SetUserCom(x)		((x)->flags |= FLAGS_USERCOM)
#define SetEService(x)		((x)->flags |= FLAGS_ESERVICE)
#define SetBService(x)		((x)->flags |= FLAGS_BSERVICE)
#define SetCService(x)		((x)->flags |= FLAGS_CSERVICE)
#define SetSBot(x)		((x)->flags |= FLAGS_SBOT)
#define SetCoder(x)		((x)->flags |= FLAGS_CODER)
#define SetHelper(x)		((x)->flags |= FLAGS_HELPER)
#define SetRCentral(x)		((x)->flags |= FLAGS_RCENTRAL)
#define SetBackupOp(x)		((x)->flags |= FLAGS_BACKUPOP)
#define SetWebCom(x)		((x)->flags |= FLAGS_WEBCOM)
#define SetRegged(x)		((x)->flags |= FLAGS_REGGED)
#define SetVlined(x)		((x)->flags |= FLAGS_VLINED)

#define ClearOper(x)		((x)->flags &= ~FLAGS_OPER)
#define ClearGod(x)		((x)->flags &= ~FLAGS_GODMODE)
#define ClearLocOp(x)		((x)->flags &= ~FLAGS_LOCOP)
#define ClearInvisible(x)	((x)->flags &= ~FLAGS_INVISIBLE)
#define ClearWallops(x)		((x)->flags &= ~FLAGS_WALLOP)
#define ClearDebug(x)           ((x)->flags &= ~FLAGS_DEBUG)
#define ClearDNS(x)		((x)->flags &= ~FLAGS_DOINGDNS)
#define ClearAuth(x)		((x)->flags &= ~FLAGS_AUTH)
#define ClearSocks(x)		((x)->flags &= ~FLAGS_SOCKS)
#define ClearWrSocks(x)		((x)->flags &= ~FLAGS_WRSOCKS)
#define ClearAccess(x)		((x)->flags &= ~FLAGS_CHKACCESS)
#define ClearPing(x)		((x)->flags &= ~FLAGS_PING)
#define ClearAskedPing(x)	((x)->flags &= ~FLAGS_ASKEDPING)
#define ClearBurst(x)		((x)->flags &= ~FLAGS_BURST)
#define ClearBurstAck(x)	((x)->flags &= ~FLAGS_BURST_ACK)
#define ClearBot(x)		((x)->flags &= ~FLAGS_BOT)
#define ClearNoPriv(x)		((x)->flags &= ~FLAGS_NOPRIVATES)
#define ClearNoCTCP(x)		((x)->flags &= ~FLAGS_NOCTCPDCC)
#define ClearHideHost(x)	((x)->flags &= ~FLAGS_HIDEHOST)
#define ClearCoAdmin(x)		((x)->flags &= ~FLAGS_COADMIN)
#define ClearAdmin(x)		((x)->flags &= ~FLAGS_ADMIN)
#define ClearUserCom(x)		((x)->flags &= ~FLAGS_USERCOM)
#define ClearEService(x)	((x)->flags &= ~FLAGS_ESERVICE)
#define ClearBService(x)	((x)->flags &= ~FLAGS_BSERVICE)
#define ClearCService(x)	((x)->flags &= ~FLAGS_CSERVICE)
#define ClearSBot(x)		((x)->flags &= ~FLAGS_SBOT)
#define ClearCoder(x)		((x)->flags &= ~FLAGS_CODER)
#define ClearHelper(x)		((x)->flags &= ~FLAGS_HELPER)
#define ClearRCentral(x)	((x)->flags &= ~FLAGS_RCENTRAL)
#define ClearBackupOp(x)	((x)->flags &= ~FLAGS_BACKUPOP)
#define ClearWebCom(x)		((x)->flags &= ~FLAGS_WEBCOM)
#define ClearRegged(x)		((x)->flags &= ~FLAGS_REGGED)
#define ClearVlined(x)		((x)->flags &= ~FLAGS_VLINED)


/* used for async dns values */

#define ASYNC_NONE	0
#define ASYNC_CLIENT	1
#define ASYNC_CONNECT	2
#define ASYNC_CONF	3
#define ASYNC_PING	4

/* server notice stuff */

#define SNO_ADD		1
#define SNO_DEL		2
#define SNO_SET		3
				/* DON'T CHANGE THESE VALUES ! */
				/* THE CLIENTS DEPEND ON IT  ! */
#define SNO_OLDSNO	0x1	/* unsorted old messages */
#define SNO_SERVKILL	0x2	/* server kills (nick collisions) */
#define SNO_OPERKILL	0x4	/* oper kills */
#define SNO_HACK2	0x8	/* desyncs */
#define SNO_HACK3	0x10	/* temporary desyncs */
#define SNO_UNAUTH	0x20	/* unauthorized connections */
#define SNO_TCPCOMMON	0x40	/* common TCP or socket errors */
#define SNO_TOOMANY	0x80	/* too many connections */
#define SNO_HACK4	0x100	/* Uworld actions on channels */
#define SNO_GLINE	0x200	/* glines */
#define SNO_NETWORK	0x400	/* net join/break, etc */
#define SNO_IPMISMATCH	0x800	/* IP mismatches */
#define SNO_THROTTLE	0x1000	/* host throttle add/remove notices */
#define SNO_OLDREALOP	0x2000	/* old oper-only messages */
#define SNO_CONNEXIT	0x4000	/* client connect/exit (ugh) */

#define SNO_ALL		0x7fff	/* Don't make it larger then significant,
				 * that looks nicer */

#define SNO_USER	(SNO_ALL & ~SNO_OPER)

#define SNO_DEFAULT (SNO_NETWORK|SNO_OPERKILL|SNO_GLINE)
#define SNO_OPERDEFAULT (SNO_DEFAULT|SNO_HACK2|SNO_HACK4|SNO_THROTTLE|SNO_OLDSNO|SNO_CONNEXIT)
#define SNO_OPER (SNO_CONNEXIT|SNO_OLDREALOP)
#define SNO_NOISY (SNO_SERVKILL|SNO_UNAUTH)

/*
 * simple defines to differentiate between a tty and socket for
 * add_connection()  -Simon
 */

#define ADCON_TTY 0
#define ADCON_SOCKET 1

/*=============================================================================
 * Proto types
 */

extern int setsnomask(aClient *cptr, snomask_t newmask, int what);
extern snomask_t umode_make_snomask(snomask_t oldmask, char *arg, int what);
extern int connect_server(aConfItem *aconf, aClient *by, struct hostent *hp);
extern void report_error(char *text, aClient *cptr);
extern int inetport(aClient *cptr, char *name, unsigned short int port);
extern int add_listener(aConfItem *aconf);
extern void close_listeners(void);
extern void init_sys(void);
extern void write_pidfile(void);
extern enum AuthorizationCheckResult check_client(aClient *cptr);
extern int check_server(aClient *cptr);
extern void close_connection(aClient *cptr);
extern int get_sockerr(aClient *cptr);
extern void set_non_blocking(int fd, aClient *cptr);
extern aClient *add_connection(aClient *cptr, int fd, int type);
extern int read_message(time_t delay);
extern void get_my_name(aClient *cptr, char *name, size_t len);
extern int setup_ping(void);

extern int highest_fd, resfd;
extern unsigned int readcalls;
extern aClient *loc_clients[MAXCONNECTIONS];
#ifdef VIRTUAL_HOST
extern struct sockaddr_in vserv;
#endif

#endif /* S_BSD_H */
